@@ -1,10 +1,10 @@
 import sys
 
+from .applications import Dispatcher, Index
+from .config import Config, here
 from .utils import State
-from .applications import Index
-from .config import here, Config
 
-__all__ = ["Index", "Config", "g"]
+__all__ = ["Index", "Dispatcher", "Config", "g"]
 
 # Current working directory first
 sys.path.insert(0, here)
