@@ -1,19 +1,17 @@
-import os
-import time
-import signal
 import logging
+import os
+import signal
 import subprocess
-from typing import List, Union
+import sys
+import time
 from multiprocessing import cpu_count
+from typing import List, Union
 
 import click
 
-from .utils import import_module
-from .config import here, LOG_LEVELS, Config
-
-
 from .__version__ import __version__
-
+from .config import LOG_LEVELS, Config, here
+from .utils import import_module
 
 config = Config()
 
@@ -41,13 +39,13 @@ def execute(command: Union[List[str], str]) -> int:
 
 
 @click.group(help=f"Index.py {__version__}")
-def main():
+def index_cli():
     # set index logger level
     indexpy_logger = logging.getLogger("indexpy")
     indexpy_logger.setLevel(LOG_LEVELS[config.LOG_LEVEL])
 
 
-@main.command(help="use only uvicorn to deploy")
+@index_cli.command(help="use uvicorn to run Index.py")
 @click.argument("application", default=lambda: config.APP)
 def serve(application):
     import uvicorn
@@ -63,39 +61,57 @@ def serve(application):
     )
 
 
-@main.command(help="use uvicorn to deploy by gunicorn")
+@click.group(help="use gunicorn to run Index.py")
+def gunicorn():
+    pass
+
+
+@gunicorn.command()
 @click.option("--workers", "-w", default=cpu_count())
+@click.option("--worker-class", "-k", default="uvicorn.workers.UvicornWorker")
 @click.option("--daemon", "-d", default=False, is_flag=True)
 @click.option(
     "--configuration",
     "-c",
     type=click.Path(exists=True, file_okay=True, dir_okay=False, readable=True),
 )
-@click.argument("method", type=click.Choice(["start", "stop", "reload"]))
 @click.argument("application", default=lambda: config.APP)
-def gunicorn(workers, daemon, configuration, method, application):
-    if method == "start":
-        command = [
-            "gunicorn -k uvicorn.workers.UvicornWorker",
-            f"--bind {config.HOST}:{config.PORT}",
-            f"--chdir {here}",
-            f"--workers {workers}",
-            f"--pid {os.path.join(here, '.pid')}",
-            f"--log-level {config.LOG_LEVEL}",
-        ]
-        if daemon:
-            command.append("-D --log-file log.index")
-        if config.AUTORELOAD:
-            command.append("--reload")
-        if configuration:
-            command.append("-c " + configuration)
-        command.append(application)
+def start(workers, worker_class, daemon, configuration, application):
+    from gunicorn.app.wsgiapp import run
 
-        execute(command)
-    elif method == "stop":
-        execute(["kill -TERM", "`cat .pid`"])
-    elif method == "reload":
-        execute(["kill -HUP", "`cat .pid`"])
+    command = (
+        "gunicorn"
+        + f" -k {worker_class}"
+        + f" --bind {config.HOST}:{config.PORT}"
+        + f" --chdir {here}"
+        + f" --workers {workers}"
+        + f" --pid {os.path.join(here, '.pid')}"
+        + f" --log-level {config.LOG_LEVEL}"
+    )
+    args = command.split(" ")
+    if daemon:
+        args.extend("-D --log-file log.index".split(" "))
+    if config.AUTORELOAD:
+        args.append("--reload")
+    if configuration:
+        args.append("-c")
+        args.append(configuration.strip())
+    args.append(application)
 
+    sys.argv = args
+    run()
+
+
+@gunicorn.command("")
+def stop():
+    execute(["kill -TERM", "`cat .pid`"])
+
+
+@gunicorn.command()
+def reload():
+    execute(["kill -HUP", "`cat .pid`"])
+
+
+index_cli.add_command(gunicorn, "gunicorn")
 
 import_module("commands")
