@@ -3,25 +3,16 @@ import json
 import typing
 from http import cookies as http_cookies
 
-from starlette.datastructures import (
-    Address,
-    URL,
-    Headers,
-    QueryParams,
-    State,
-)
+from starlette.datastructures import URL, Address, Headers, QueryParams, State
 from starlette.formparsers import (
     FormData,
-    MultiPartParser,
     FormParser,
+    MultiPartParser,
     parse_options_header,
 )
-from starlette.requests import (
-    SERVER_PUSH_HEADERS_TO_COPY,
-    ClientDisconnect,
-)
+from starlette.requests import SERVER_PUSH_HEADERS_TO_COPY, ClientDisconnect
 
-from ..types import Scope, Send, Receive, Message
+from ..types import Message, Receive, Scope, Send
 from ..utils import cached_property
 
 
@@ -71,6 +62,11 @@ class HTTPConnection(typing.Mapping):
     def __len__(self) -> int:
         return len(self.scope)
 
+    @cached_property
+    def client(self) -> Address:
+        host, port = self.scope.get("client") or (None, None)
+        return Address(host=host, port=port)
+
     @cached_property
     def url(self) -> URL:
         return URL(scope=self.scope)
@@ -107,9 +103,11 @@ class HTTPConnection(typing.Mapping):
         return cookies
 
     @cached_property
-    def client(self) -> Address:
-        host, port = self.scope.get("client") or (None, None)
-        return Address(host=host, port=port)
+    def session(self) -> typing.Dict[str, typing.Any]:
+        assert (
+            "session" in self.scope
+        ), "`starlette.middleware.sessions.SessionMiddleware` must be installed to access request.session"
+        return self.scope["session"]
 
     @cached_property
     def state(self) -> State:
