@@ -1,21 +1,15 @@
 import os
 from copy import deepcopy
-from typing import cast, List, Dict, Any, Sequence
+from typing import Any, Dict, List, Sequence, cast
 
 from starlette.endpoints import Request, Response
 
-from ..types import Scope, Receive, Send, TypedDict, Literal
-from ..routing import NoMatchFound
-from ..http.responses import (
-    JSONResponse,
-    YAMLResponse,
-    HTMLResponse,
-)
-from ..applications import Index, IndexFile
-
+from ..applications import Index
+from ..http.responses import HTMLResponse, JSONResponse, YAMLResponse
+from ..routing import HttpRoute
+from ..types import Literal, TypedDict
 from .schema import schema_parameters, schema_request_body, schema_response
 
-
 Tag = TypedDict("Tag", {"description": str, "paths": Sequence[str]})
 
 
@@ -59,22 +53,11 @@ class OpenAPI:
     def _generate_paths(self, app: Index) -> Dict[str, Any]:
         result = {}
 
-        for path_format, path_convertors, endpoint in app.router.http_routes.values():
+        for path_format, endpoint in app.router.http_tree.iterator():
             path_docs = self._generate_path(endpoint, path_format)
             if path_docs:
                 result[path_format] = path_docs
 
-        for index_file_app in reversed(
-            [subapp for _, subapp in app.mount_apps if isinstance(subapp, IndexFile)]
-        ):
-            for view, path in index_file_app.get_views():
-                if not hasattr(view, "HTTP"):
-                    continue
-                viewclass = getattr(view, "HTTP")
-                path_docs = self._generate_path(viewclass, path)
-                if path_docs:
-                    result[path] = path_docs
-
         return result
 
     def _generate_path(self, view: Any, path: str) -> Dict[str, Any]:
@@ -154,34 +137,29 @@ class OpenAPI:
 
         return openapi
 
-    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
-        if scope["type"] != "http" or scope["path"] not in ("/", "/get"):
-            raise NoMatchFound()
-        request = Request(scope, receive, send)
+    @property
+    def routes(self) -> List[HttpRoute]:
+        async def template(request: Request) -> Response:
+            if self.html_template:
+                return HTMLResponse(self.html_template)
 
-        if scope["path"] == "/get":
-            handler = getattr(self, "docs")
-        elif scope["path"] == "/":
-            handler = getattr(self, "template")
-        else:
-            raise NoMatchFound()
-        response = await handler(request)
-        await response(scope, receive, send)
+            with open(
+                os.path.join(
+                    os.path.dirname(os.path.abspath(__file__)), "template.html"
+                )
+            ) as file:
+                DEFAULT_TEMPLATE = file.read()
+            return HTMLResponse(DEFAULT_TEMPLATE)
 
-    async def template(self, request: Request) -> Response:
-        if self.html_template:
-            return HTMLResponse(self.html_template)
+        async def docs(request: Request) -> Response:
+            openapi = self.create_docs(request)
+            media_type = request.query_params.get("type") or self.media_type
 
-        with open(
-            os.path.join(os.path.dirname(os.path.abspath(__file__)), "template.html")
-        ) as file:
-            DEFAULT_TEMPLATE = file.read()
-        return HTMLResponse(DEFAULT_TEMPLATE)
+            if media_type == "json":
+                return JSONResponse(openapi)
+            return YAMLResponse(openapi)
 
-    async def docs(self, request: Request) -> Response:
-        openapi = self.create_docs(request)
-        media_type = request.query_params.get("type") or self.media_type
-
-        if media_type == "json":
-            return JSONResponse(openapi)
-        return YAMLResponse(openapi)
+        return [
+            HttpRoute("/", template, name=None, method="get"),
+            HttpRoute("/get", docs, name=None, method="get"),
+        ]
