@@ -1,34 +1,27 @@
-import typing
 import copy
+import importlib
+import os
+import typing
+from dataclasses import InitVar, asdict, dataclass
 from functools import update_wrapper, wraps
-from types import FunctionType
-from dataclasses import dataclass, InitVar
+from pathlib import Path
 
-from indexpy.types import Literal, ASGIApp, Scope, Receive, Send
-from indexpy.utils import superclass
 from indexpy.concurrency import complicating
-from indexpy.http.view import bound_params, parse_params, only_allow
 from indexpy.http.responses import convert
+from indexpy.http.view import only_allow
+from indexpy.types import ASGIApp, Literal, Receive, Scope, Send
+from indexpy.utils import superclass
 
 from .convertors import Convertor, compile_path
 from .tree import RadixTree
 
-__all__ = [
-    "Routes",
-    "SubRoutes",
-    "HttpRoute",
-    "SocketRoute",
-    "NoMatchFound",
-    "NoRouteFound",
-]
-
 
 def request_response(view: typing.Any) -> ASGIApp:
     @wraps(view)
     async def _(scope: Scope, receive: Receive, send: Send) -> None:
         current_app = scope["app"]
         request = current_app.factory_class.http(scope, receive, send)
-        response = convert(await (await parse_params(view, request))(request))
+        response = convert(await view(request))
         await response(scope, receive, send)
 
     return _
@@ -44,6 +37,23 @@ def websocket_session(view: typing.Any) -> ASGIApp:
     return _
 
 
+def subpath_asgi(path_prefix: str, asgi: ASGIApp) -> ASGIApp:
+    assert path_prefix.startswith("/"), "path_prefix must be start with '/'"
+    assert not path_prefix.endswith("/"), "path_prefix can't end with '/'"
+
+    @wraps(asgi)
+    async def _(scope: Scope, receive: Receive, send: Send) -> None:
+        path = scope["path"]
+        root_path = scope.get("root_path", "")
+
+        subscope = copy.copy(scope)
+        subscope["path"] = path[len(path_prefix) :]
+        subscope["root_path"] = root_path + path_prefix
+        await asgi(subscope, receive, send)
+
+    return _
+
+
 class NoMatchFound(Exception):
     """
     Raised by `.search(path)` if no matching route exists.
@@ -61,7 +71,6 @@ class BaseRoute:
     path: str
     endpoint: typing.Any
     name: typing.Optional[str] = ""
-    is_asgi_app: bool = False
 
     def extend_middlewares(self, routes: typing.List["BaseRoute"]) -> None:
         raise NotImplementedError()
@@ -88,9 +97,7 @@ class HttpRoute(BaseRoute):
         ):
             if method == "":
                 raise ValueError("View function must be marked with method")
-            self.endpoint = only_allow(
-                method, bound_params(typing.cast(FunctionType, self.endpoint)),
-            )
+            self.endpoint = only_allow(method, self.endpoint)
         else:
             if hasattr(self.endpoint, "__method__") and (
                 method.upper() not in (getattr(self.endpoint, "__method__"), "")
@@ -100,9 +107,9 @@ class HttpRoute(BaseRoute):
                 raise ValueError("View class can't be marked with method")
 
     def extend_middlewares(self, routes: typing.List[BaseRoute]) -> None:
-        if hasattr(routes, "http_middlewares"):
+        if getattr(routes, "_http_middlewares", None):
             endpoint = self.endpoint
-            for middleware in getattr(routes, "http_middlewares"):
+            for middleware in getattr(routes, "_http_middlewares"):
                 endpoint = middleware(endpoint)
             self.endpoint = update_wrapper(endpoint, self.endpoint)
 
@@ -110,9 +117,28 @@ class HttpRoute(BaseRoute):
 @dataclass
 class SocketRoute(BaseRoute):
     def extend_middlewares(self, routes: typing.List[BaseRoute]) -> None:
-        if hasattr(routes, "socket_middlewares"):
+        if getattr(routes, "_socket_middlewares", None):
             endpoint = self.endpoint
-            for middleware in getattr(routes, "socket_middlewares"):
+            for middleware in getattr(routes, "_socket_middlewares"):
+                endpoint = middleware(endpoint)
+            self.endpoint = update_wrapper(endpoint, self.endpoint)
+
+
+@dataclass
+class ASGIRoute(BaseRoute):
+    # This is mypy error
+    type: typing.Container[Literal["http", "websocket"]] = ("http", "websocket")  # type: ignore
+    root_path: InitVar[str] = ""
+
+    def __post_init__(self, root_path: str) -> None:  # type: ignore
+        super().__post_init__()
+        if root_path:
+            self.endpoint = subpath_asgi(root_path, self.endpoint)
+
+    def extend_middlewares(self, routes: typing.List[BaseRoute]) -> None:
+        if hasattr(routes, "_asgi_middlewares"):
+            endpoint = self.endpoint
+            for middleware in getattr(routes, "_asgi_middlewares"):
                 endpoint = middleware(endpoint)
             self.endpoint = update_wrapper(endpoint, self.endpoint)
 
@@ -132,18 +158,17 @@ class RouteRegisterMixin:
                 route.extend_middlewares(routes)
 
             if isinstance(routes, SubRoutes):
-                self.append(
-                    route.__class__(
-                        path=routes.prefix + route.path,
-                        endpoint=route.endpoint,
-                        name=route.name,
-                    )
-                )
-            else:
-                self.append(route)
+                data = asdict(route)
+                data["path"] = routes.prefix + data["path"]
+                route = route.__class__(**data)
+
+            if getattr(routes, "namespace", None) and route.name:
+                route.name = getattr(routes, "namespace") + ":" + route.name
+
+            self.append(route)
 
     def http(
-        self, path: str, *, name: str = "", method: str = "", is_asgi_app: bool = False
+        self, path: str, *, name: str = "", method: str = ""
     ) -> typing.Callable[[T], T]:
         """
         shortcut for `self.append(HttpRoute(path, endpoint, name, method))`
@@ -154,14 +179,12 @@ class RouteRegisterMixin:
         """
 
         def register(endpoint: T) -> T:
-            self.append(HttpRoute(path, endpoint, name, is_asgi_app, method))
+            self.append(HttpRoute(path, endpoint, name, method))
             return endpoint
 
         return register
 
-    def websocket(
-        self, path: str, *, name: str = "", is_asgi_app: bool = False
-    ) -> typing.Callable[[T], T]:
+    def websocket(self, path: str, *, name: str = "") -> typing.Callable[[T], T]:
         """
         shortcut for `self.append(SocketRoute(path, endpoint, name))`
 
@@ -171,7 +194,29 @@ class RouteRegisterMixin:
         """
 
         def register(endpoint: T) -> T:
-            self.append(SocketRoute(path, endpoint, name, is_asgi_app))
+            self.append(SocketRoute(path, endpoint, name))
+            return endpoint
+
+        return register
+
+    def asgi(
+        self,
+        path: str,
+        *,
+        name: str = "",
+        type: typing.Container[Literal["http", "websocket"]] = ("http", "websocket"),  # type: ignore
+        root_path: str = "",
+    ) -> typing.Callable[[T], T]:
+        """
+        shortcut for `self.append(ASGIRoute(path, endpoint, name, type))`
+
+        example:
+            @routes.asgi("/path", name="endpoint-name")
+            class Endpoint(HTTPView): ...
+        """
+
+        def register(endpoint: T) -> T:
+            self.append(ASGIRoute(path, endpoint, name, type, root_path))
             return endpoint
 
         return register
@@ -180,15 +225,19 @@ class RouteRegisterMixin:
 class Routes(typing.List[BaseRoute], RouteRegisterMixin):
     def __init__(
         self,
-        *iterable: typing.Union["BaseRoute", "Routes"],
+        *iterable: typing.Union["BaseRoute", "Routes", "FileRoutes"],
+        namespace: str = "",
         http_middlewares: typing.List[typing.Any] = [],
         socket_middlewares: typing.List[typing.Any] = [],
+        asgi_middlewares: typing.List[typing.Any] = [],
     ) -> None:
         super().__init__()
-        self.http_middlewares = copy.copy(http_middlewares)
-        self.socket_middlewares = copy.copy(socket_middlewares)
+        self.namespace = namespace
+        self._http_middlewares = copy.copy(http_middlewares)
+        self._socket_middlewares = copy.copy(socket_middlewares)
+        self._asgi_middlewares = copy.copy(asgi_middlewares)
         for route in iterable:
-            if not isinstance(route, Routes):
+            if not isinstance(route, typing.List):
                 self.append(route)
             else:
                 self.extend(route)
@@ -215,16 +264,8 @@ class Routes(typing.List[BaseRoute], RouteRegisterMixin):
                     response = await endpoint(request)
                     return response
                 return wrapper
-        or
-            def middleware(endpoint):
-                async def wrapper(request):
-                    response = await endpoint(request)
-                    return response
-                return wrapper
-
-            routes.http_middleware(middleware)
         """
-        self.http_middlewares.append(middleware)
+        self._http_middlewares.append(middleware)
         return middleware
 
     def socket_middleware(self, middleware: T) -> T:
@@ -237,15 +278,22 @@ class Routes(typing.List[BaseRoute], RouteRegisterMixin):
                 async def wrapper(websocket):
                     await endpoint(websocket)
                 return wrapper
-        or
-            def middleware(endpoint):
-                async def wrapper(websocket):
-                    await endpoint(websocket)
-                return wrapper
-
-            routes.socket_middleware(middleware)
         """
-        self.socket_middlewares.append(middleware)
+        self._socket_middlewares.append(middleware)
+        return middleware
+
+    def asgi_middleware(self, middleware: T) -> T:
+        """
+        append middleware in routes
+
+        example:
+            @routes.asgi_middleware
+            def middleware(endpoint):
+                async def wrapper(scope, receive, send):
+                    await endpoint(scope, receive, send)
+                return wrapper
+        """
+        self._asgi_middlewares.append(middleware)
         return middleware
 
 
@@ -253,21 +301,75 @@ class SubRoutes(Routes):
     def __init__(
         self,
         prefix: str,
-        routes: Routes = typing.cast(Routes, []),
+        routes: typing.List[BaseRoute] = [],
         *,
+        namespace: str = "",
         http_middlewares: typing.List[typing.Any] = [],
         socket_middlewares: typing.List[typing.Any] = [],
+        asgi_middlewares: typing.List[typing.Any] = [],
     ) -> None:
         if not prefix.startswith("/") or prefix.endswith("/"):
             raise ValueError("Mount prefix cannot end with '/' and must start with '/'")
         self.prefix = prefix
         super().__init__(
             *routes,
+            namespace=namespace,
             http_middlewares=http_middlewares,
             socket_middlewares=socket_middlewares,
+            asgi_middlewares=asgi_middlewares,
         )
 
 
+class FileRoutes(typing.List[BaseRoute]):
+    def __init__(
+        self,
+        module_name: str,
+        *,
+        namespace: str = "",
+        allow_underline: bool = False,
+        suffix: str = "",
+    ) -> None:
+        dirpath = Path(
+            os.path.abspath(importlib.import_module(module_name).__file__)
+        ).parent
+        assert dirpath.name == module_name
+
+        for pypath in dirpath.glob("**/*.py"):
+            relpath = str(pypath.relative_to(dirpath)).replace("\\", "/")[:-3]
+
+            path_list = relpath.split("/")
+            path_list.insert(0, module_name)
+
+            url_path = "/" + relpath
+            if not allow_underline:
+                url_path = url_path.replace("_", "-")
+            if url_path.endswith("/index"):
+                url_path = url_path[: -len("index")]
+            else:
+                url_path = url_path + suffix
+
+            module = importlib.import_module(".".join(path_list))
+            url_name = getattr(module, "name", None)
+            get_response = getattr(module, "HTTP", None)
+            serve_socket = getattr(module, "Socket", None)
+            if get_response:
+                for deep in range(len(path_list), 0, -1):
+                    _module = importlib.import_module(".".join(path_list[:deep]))
+                    if not hasattr(_module, "HTTPMiddleware"):
+                        continue
+                    get_response = getattr(_module, "HTTPMiddleware")(get_response)
+                self.append(HttpRoute(url_path, get_response, url_name))
+            if serve_socket:
+                for deep in range(len(path_list), 0, -1):
+                    _module = importlib.import_module(".".join(path_list[:deep]))
+                    if not hasattr(_module, "SocketMiddleware"):
+                        continue
+                    serve_socket = getattr(_module, "SocketMiddleware")(serve_socket)
+                self.append(SocketRoute(url_path, serve_socket, url_name))
+
+        self.namespace = namespace
+
+
 class Router(RouteRegisterMixin):
     def __init__(self, routes: typing.List[BaseRoute] = list()) -> None:
         self.http_tree = RadixTree()
@@ -282,31 +384,63 @@ class Router(RouteRegisterMixin):
 
         self.extend(routes)
 
+    def _append(
+        self,
+        path: str,
+        endpoint: ASGIApp,
+        name: typing.Optional[str],
+        radix_tree: RadixTree,
+        routes: typing.Dict,
+    ) -> None:
+        if name in routes:
+            raise ValueError(f"Duplicate route name: {name}")
+
+        radix_tree.append(path, endpoint)
+        path_format, path_convertors = compile_path(path)
+
+        if name:  # name not in ("", None)
+            routes[name] = (path_format, path_convertors, endpoint)
+
     def append(self, route: BaseRoute) -> None:
         if isinstance(route, HttpRoute):
-            radix_tree = self.http_tree
-            routes = self.http_routes
-            if not route.is_asgi_app:
-                route.endpoint = request_response(route.endpoint)
+            self._append(
+                route.path,
+                request_response(route.endpoint),
+                route.name,
+                self.http_tree,
+                self.http_routes,
+            )
         elif isinstance(route, SocketRoute):
-            radix_tree = self.websocket_tree
-            routes = self.websocket_routes
-            if not route.is_asgi_app:
-                route.endpoint = websocket_session(route.endpoint)
+            self._append(
+                route.path,
+                websocket_session(route.endpoint),
+                route.name,
+                self.websocket_tree,
+                self.websocket_routes,
+            )
+        elif isinstance(route, ASGIRoute):
+            if "http" in route.type:
+                self._append(
+                    route.path,
+                    route.endpoint,
+                    route.name,
+                    self.http_tree,
+                    self.http_routes,
+                )
+            if "websocket" in route.type:
+                self._append(
+                    route.path,
+                    route.endpoint,
+                    route.name,
+                    self.websocket_tree,
+                    self.websocket_routes,
+                )
         else:
             raise TypeError(
-                "Need type: `HttpRoute` or `SocketRoute`,"
+                "Need type: `ASGIRoute`, `HttpRoute` or `SocketRoute`,"
                 + f" but got type: {type(route)}"
             )
 
-        radix_tree.append(route.path, route.endpoint)
-        path_format, path_convertors = compile_path(route.path)
-
-        if route.name in routes:
-            raise ValueError(f"Duplicate route name: {route.name}")
-        if route.name:
-            routes[route.name] = (path_format, path_convertors, route.endpoint)
-
     def search(
         self, protocol: Literal["http", "websocket"], path: str
     ) -> typing.Tuple[typing.Dict[str, typing.Any], ASGIApp]:
