@@ -1,6 +1,6 @@
 import re
-from typing import Optional, Union, Dict, List, Tuple, Pattern, Any
 from dataclasses import dataclass, field
+from typing import Any, Dict, Generator, List, Optional, Pattern, Tuple, Union
 
 from ..types import ASGIApp
 from .convertors import Convertor, PathConvertor, compile_path
@@ -111,8 +111,10 @@ def search(
 
         path = path[length:]
         if not path:  # path == "", found the first suitable route
-            for name in set(params.keys()) - set(point.param_convertors.keys()):
-                del params[name]
+            [
+                params.pop(name)
+                for name in set(params.keys()) - set(point.param_convertors.keys())
+            ]
             return params, point
 
         for node in point.next_nodes:
@@ -155,3 +157,16 @@ class RadixTree:
             },
             node.endpoint,
         )
+
+    def iterator(self) -> Generator[Tuple[str, ASGIApp], None, None]:
+        stack: List[Tuple[str, TreeNode]] = [(self.root.characters, self.root)]
+
+        while stack:
+            characters, point = stack.pop(len(stack) - 1)
+            for node in point.next_nodes:
+                if node.re_pattern is not None:
+                    stack.append((f"{characters}{{{node.characters}}}", node))
+                else:
+                    stack.append((f"{characters}{node.characters}", node))
+            if point.endpoint:
+                yield characters, point.endpoint
